// Defined to disable deprecated functions warning in Visual C++ 2005
#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>
#include <stdio.h>

HINSTANCE WRG315API=NULL;
DWORD WRG315API_Count=0;

typedef void (__stdcall *FNCDSPCallback)(void *Target);

typedef int (__stdcall *FNCOpenRadioDevice)(int iDeviceNum);
typedef BOOL (__stdcall *FNCCloseRadioDevice)(int hRadio);
typedef BOOL (__stdcall *FNCdspBoot)(int hRadio,DWORD DSPHandle,DWORD Mask,const void *Data,DWORD Size);
typedef BOOL (__stdcall *FNCdspClose)(int hRadio,DWORD DSPHandle);
typedef DWORD (__stdcall *FNCdspOpen)(int hRadio,DWORD Mask,FNCDSPCallback CallbackFunc,void *CallbackTarget);
typedef BOOL (__stdcall *FNCdspRead)(int hRadio,DWORD DSPHandle,DWORD Mask,DWORD Offset,void *lpData,DWORD Size);
typedef BOOL (__stdcall *FNCdspReset)(int hRadio,DWORD DSPHandle,DWORD Mask);
typedef BOOL (__stdcall *FNCdspWrite)(int hRadio,DWORD DSPHandle,DWORD Mask,DWORD Offset,const void *lpData,DWORD Size);

FNCOpenRadioDevice OpenRadioDevice=NULL;
FNCCloseRadioDevice CloseRadioDevice=NULL;
FNCdspBoot dspBoot=NULL;
FNCdspClose dspClose=NULL;
FNCdspOpen dspOpen=NULL;
FNCdspRead dspRead=NULL;
FNCdspReset dspReset=NULL;
FNCdspWrite dspWrite=NULL;

void __fastcall APIUnload(void)
{
	//  If the API has not been loaded it simply returns. Else if the API has been
	// loaded more than once, the special counter is decremented. Only if the
	// special counter drops back to 0 the library is actually freed.
	if (!WRG315API_Count || --WRG315API_Count!=0) return;
	FreeLibrary(WRG315API);
	WRG315API=NULL;
	CloseRadioDevice=NULL;
	OpenRadioDevice=NULL;
	dspBoot=NULL;
	dspClose=NULL;
	dspOpen=NULL;
	dspRead=NULL;
	dspReset=NULL;
	dspWrite=NULL;
}

BOOL __fastcall APILoad(void)
{
	//  If the API has already been loaded, only increment the special counter and
	// return with success.
	if (WRG315API_Count) { WRG315API_Count++;return TRUE; }
	//  Tries to load the API. If this fails it returns with error.
	if ((WRG315API=LoadLibrary("WRG315API"))==NULL) return FALSE;
	//  The API has been successfully loaded. Now it has to connect to its
	// exported functions.
	CloseRadioDevice=(FNCCloseRadioDevice)GetProcAddress(WRG315API,"CloseRadioDevice");
	OpenRadioDevice=(FNCOpenRadioDevice)GetProcAddress(WRG315API,"OpenRadioDevice");
	dspBoot=(FNCdspBoot)GetProcAddress(WRG315API,"dspBoot");
	dspClose=(FNCdspClose)GetProcAddress(WRG315API,"dspClose");
	dspOpen=(FNCdspOpen)GetProcAddress(WRG315API,"dspOpen");
	dspRead=(FNCdspRead)GetProcAddress(WRG315API,"dspRead");
	dspReset=(FNCdspReset)GetProcAddress(WRG315API,"dspReset");
	dspWrite=(FNCdspWrite)GetProcAddress(WRG315API,"dspWrite");
	//  If all functions have been found it increments the special counter. If not
	// the library is unloaded.
	WRG315API_Count++;
	if (CloseRadioDevice && OpenRadioDevice && dspBoot && dspClose && dspOpen && dspRead && dspReset && dspWrite) return TRUE;
	//  As the library has not been completely loaded, it clears any already
	// initialized data.
	APIUnload();
	return FALSE;
}

//
//  Various constants that affect the overall behaviour of the receiver
#define ReceiverBufferSize      1024
#define ReceiverBuffersNum      4
#define ReceiverSamplingRate    64000

#define WR_G313_DSP_1_1         0x00000001
#define WR_G313_DSP_1_2         0x00000002

//
// structure used to pass data to the interrupt handler
typedef struct {
	int hRadio;
	DWORD DSPHandle;
} InterruptContext;

void __stdcall InterruptHandler(void *Target)
{
	InterruptContext *Ctx=(InterruptContext *)Target;
	short Buf[ReceiverBufferSize];
	WORD Code;

	//  Retrieves the interrupt code.
	dspRead(Ctx->hRadio,Ctx->DSPHandle,WR_G313_DSP_1_1,0x4000,&Code,1);
	if (Code>=1 && Code<=ReceiverBuffersNum) {
		//  Reads the IF samples.
		dspRead(Ctx->hRadio,Ctx->DSPHandle,WR_G313_DSP_1_1,(WORD)((Code-1)*ReceiverBufferSize+0x6000),Buf,ReceiverBufferSize);

		//
		//  *** here comes the signal processing ***
		//

		//  Writes the resulted samples.
		dspWrite(Ctx->hRadio,Ctx->DSPHandle,WR_G313_DSP_1_1,(WORD)((Code-1)*ReceiverBufferSize+0x5000),Buf,ReceiverBufferSize);
		//  Signals the interrupt has been processed.
		Code=0;
		dspWrite(Ctx->hRadio,Ctx->DSPHandle,WR_G313_DSP_1_1,0x4000,&Code,1);
	}
}

int main(int argc, char* argv[])
{
	InterruptContext IntCtx;

	static WORD DSPBOOT[] = {
		0x0006,0x8000,0x4000,0x0000,0x0000,0x0000,0x0000,0x0000,
		0x0001,0x0010,0x8000,0x4020,0x810F,0x8279,0x8301,0x8400,
		0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
		0x2000,0x1000,0x0000,0x0000,0x0006,0x8000,0x0001,0x4040,
		0x0000,0x93FF,0x00F0,0x1833,0x002F,0x0002,0x8000,0x0010,
		0x0A00,0x001F,0x0008,0x8000,0x0014,0x0C00,0x0030,0x8002,
		0x0080,0x9002,0x0010,0x1835,0x00CF,0x00CE,0x8000,0x0332,
		0x3C0F,0x00FC,0x3C00,0x0004,0x3C00,0x0000,0x3C00,0x0001,
		0x0202,0x008F,0x400F,0x000A,0x93FE,0x006A,0x93FE,0x005A,
		0x3402,0x0043,0x3400,0x004B,0x3400,0x0017,0x3802,0x0003,
		0x3800,0x004B,0x3800,0x0017,0x40FB,0x00F0,0x93FF,0x0030,
		0x4280,0x00F0,0x93FF,0x0060,0x0203,0x000F,0x0200,0x00CF,
		0x3C04,0x0003,0x4140,0x0000,0x93FF,0x00F0,0x7000,0x000F,
		0x0D0C,0x0090,0x0280,0x0000,0x4852,0x0000,0x9002,0x0000,
		0x4860,0x0000,0x9002,0x0010,0x4870,0x0000,0x9002,0x0020,
		0x4800,0x0010,0x9002,0x0030,0x0280,0x0000,0x3C02,0x0003,
		0x8000,0x004A,0x227A,0x000F,0x1835,0x00B0,0x0D08,0x002A,
		0x0B00,0x009F,0x1835,0x006F,0x8002,0x0090,0x9002,0x0020,
		0x8002,0x00A0,0x9002,0x0030,0x8002,0x00B0,0x9002,0x0000,
		0x8000,0x005A,0x2782,0x001F,0x1836,0x00F0,0x23C2,0x001F,
		0x9000,0x005A,0x4000,0x000A,0x9002,0x00EA,0x4200,0x000A,
		0x9002,0x00CA,0x4100,0x000A,0x9002,0x00DA,0x4000,0x000A,
		0x9002,0x00FA,0x8402,0x00D1,0x3500,0x0009,0x6000,0x0007,
		0x9002,0x0080,0x6000,0x0007,0x9002,0x0090,0x6000,0x0007,
		0x9002,0x00A0,0x6000,0x0007,0x9002,0x00B0,0x9402,0x00D1,
		0x8402,0x00C1,0x3500,0x0009,0x8002,0x004A,0x6800,0x00A7,
		0x8002,0x005A,0x6800,0x00A7,0x8002,0x006A,0x6800,0x00A7,
		0x8002,0x007A,0x6800,0x00A7,0x9402,0x00C1,0x8002,0x00EA,
		0x4000,0x0044,0x2262,0x000F,0x9002,0x00EA,0x22F2,0x009F,
		0x1839,0x0084,0x8000,0x0000,0x2A78,0x00AA,0x1839,0x0081,
		0x9002,0x00EA,0x4000,0x000A,0x93FE,0x005A,0x8002,0x00F4,
		0x2220,0x000F,0x9000,0x000A,0x2382,0x00BF,0x9002,0x00FA,
		0x400F,0x000A,0x93FE,0x005A,0x0A00,0x001F,0x0002,0x8000,
		0x0000,0x3C00,0x0003,0xFFFF
	};

	// load the G3 API library
	if (!APILoad()) {
		puts("WRG315API.DLL cannot be loaded!");
		return 0;
	}
	// open the first available radio
	IntCtx.hRadio=OpenRadioDevice(0);
	if (!IntCtx.hRadio) {
		puts("No WR-G315 device could be opened !");
		APIUnload();
		return 0;
	}

	// open the DSP
	IntCtx.DSPHandle=dspOpen(IntCtx.hRadio,WR_G313_DSP_1_1,InterruptHandler,&IntCtx);
	if (IntCtx.DSPHandle) {
		// reset the DSP
		dspReset(IntCtx.hRadio,IntCtx.DSPHandle,WR_G313_DSP_1_1);
		// boot the DSP
		dspBoot(IntCtx.hRadio,IntCtx.DSPHandle,WR_G313_DSP_1_1,DSPBOOT,sizeof(DSPBOOT)/sizeof(DSPBOOT));


		//
		// main application body comes here, the following instructions just waits for a key to be pressed
		//
		int i;
		scanf("%d",&i);


		// reset the DSP
		dspReset(IntCtx.hRadio,IntCtx.DSPHandle,WR_G313_DSP_1_1);
		// close the DSP
		dspClose(IntCtx.hRadio,IntCtx.DSPHandle);
	}
	else puts("DSP could not be opened !");

	// close the device handle
	if (CloseRadioDevice(IntCtx.hRadio)) puts("The device is closed properly");
	else puts("The device failed to close");

	// free the G3 API library
	APIUnload();
	return 0;
}
