// Defined to disable deprecated functions warning in Visual C++ 2005
#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>
#include <stdio.h>

// G3 API function type declarations
typedef int (__stdcall *FNCOpenRadioDevice)(int iDeviceNum);
typedef BOOL (__stdcall *FNCCloseRadioDevice)(int hRadio);
typedef BOOL (__stdcall *FNCSetAtten)(int hRadio, BOOL fAtten);
typedef BOOL (__stdcall *FNCSetPower)(int hRadio, BOOL fPower);
typedef BOOL (__stdcall *FNCSetAGC)(int hRadio, int iAGC);
typedef BOOL (__stdcall *FNCBlockScan)(int hRadio,DWORD *Freqs,size_t Count,int StopSquelchRaw,DWORD FeedbackTime,HWND WinHandle,DWORD Msg);
typedef BOOL (__stdcall *FNCStopBlockScan)(int hRadio);

int main(int argc, char* argv[])
{
	// load the G3 API library
	HMODULE dll=LoadLibrary("wrg315api.dll");
	if (!dll) {
		puts("WRG315API.DLL not found !"); 
		return 0;
	}

	// link G3 API functions
	FNCOpenRadioDevice OpenRadioDevice=(FNCOpenRadioDevice)GetProcAddress(dll,"OpenRadioDevice");
	FNCCloseRadioDevice CloseRadioDevice=(FNCCloseRadioDevice)GetProcAddress(dll,"CloseRadioDevice");
	FNCSetAtten SetAtten=(FNCSetAtten)GetProcAddress(dll,"SetAtten");
	FNCSetPower SetPower=(FNCSetPower)GetProcAddress(dll,"SetPower");
	FNCSetAGC SetAGC=(FNCSetAGC)GetProcAddress(dll,"SetAGC");
	FNCBlockScan BlockScan=(FNCBlockScan)GetProcAddress(dll,"BlockScan");
	FNCStopBlockScan StopBlockScan=(FNCStopBlockScan)GetProcAddress(dll,"StopBlockScan");

	// open the first available radio
	int hRadio=OpenRadioDevice(0);
	if (!hRadio) {
		puts("No G315 device could be opened !");
		FreeLibrary(dll); 
		return 0;
	}

	// set Power,AGC and Attenuator
	if (SetPower(hRadio,TRUE)) puts("The device is turned ON");
	else puts("The device failed to turn ON");
	if (SetAtten(hRadio,FALSE)) puts("The attenuator is OFF");
	else puts("The attenuator failed to turn OFF");
	if (SetAGC(hRadio,3)) puts("The AGC is FAST");
	else puts("The AGC failed to switch");

	// prepare the buffer for the blockscan from 600 to 800 kHz
	size_t i;
	DWORD Freqs[1001];  // buffer for frequencies to scan
	for (i=0;i<=1000;i++) Freqs[i]=(DWORD)(600000+200*i);

	// let's start the blockscan with 1001 frequencies, don't stop by Squelch.
	// no feedback (after a million ms), don't send us the buffers, we'll check that all
	if (BlockScan(hRadio,Freqs,1001,256,1000000,NULL,0)) puts("The block scanning started..");
	else puts("The block scanning failed to start");

	puts("Scanned data:");
	for (i=0;i<=1000;i++) {
		while (Freqs[i]>255) Sleep(10);    // the frequency is not scanned yet, we have to wait
		printf("Freq: %ld\tRAW:%ld\n",600000+i*200,Freqs[i]);
	}

	// the block scanning is over
	if (StopBlockScan(hRadio)) puts("Block scanning is stopped");
	else puts("Block scanning couldn't stop");

	// close the device handle
	if (CloseRadioDevice(hRadio)) puts("The device is closed properly");
	else puts("The device failed to close");

	// free the G3 API library
	FreeLibrary(dll);

	return 0;
}